@@ -8,7 +8,7 @@
 #endif
 
 #ifndef FALSE
-#  define FALSE (0)
+#  define FALSE (!1)
 #endif
 
 #ifndef TRUE
@@ -22,9 +22,13 @@
 	(((size) + MEM_ALIGN_SIZE-1) & ~((size_t) MEM_ALIGN_SIZE-1))
 
 #define PTR_OFFSET(ptr, offset) \
-	((void *) (((unsigned char *) (ptr)) + (offset)))
+	((void *) (((uintptr_t) (ptr)) + (offset)))
 #define CONST_PTR_OFFSET(ptr, offset) \
-	((const void *) (((const unsigned char *) (ptr)) + (offset)))
+	((const void *) (((uintptr_t) (ptr)) + (offset)))
+
+#define container_of(ptr, type, name) \
+	(type *)((uintptr_t)(ptr) - (uintptr_t)offsetof(type, name) + \
+		 COMPILE_ERROR_IF_TYPES_NOT_COMPATIBLE(ptr, &((type *) 0)->name))
 
 /* Don't use simply MIN/MAX, as they're often defined elsewhere in include
    files that are included after this file generating tons of warnings. */
@@ -35,7 +39,7 @@
    sizeof(size_t) == sizeof(void *) and they're both the largest datatypes
    that are allowed to be used. so, long long isn't safe with these. */
 #define POINTER_CAST(i) \
-	((void *) ((char *) NULL + (i)))
+	((void *) (((uintptr_t)NULL) + (i)))
 #define POINTER_CAST_TO(p, type) \
 	((type) ((const char *) (p) - (const char *) NULL))
 
@@ -146,6 +150,11 @@
 #else
 #  define ATTR_RETURNS_NONNULL
 #endif
+#ifdef HAVE_ATTR_DEPRECATED
+#  define ATTR_DEPRECATED(str) __attribute__((deprecated(str)))
+#else
+#  define ATTR_DEPRECATED(str)
+#endif
 
 /* Macros to provide type safety for callback functions' context parameters */
 #ifdef HAVE_TYPE_CHECKS
@@ -158,7 +167,7 @@
 
 #if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ > 0)) && !defined(__cplusplus)
 #  define COMPILE_ERROR_IF_TRUE(condition) \
-	(sizeof(char[1 - 2 * !!(condition)]) - 1)
+	(sizeof(char[1 - 2 * ((condition) ? 1 : 0)]) - 1)
 #else
 #  define COMPILE_ERROR_IF_TRUE(condition) 0
 #endif
@@ -177,8 +186,8 @@
 #endif
 
 #if __GNUC__ > 2
-#  define unlikely(expr) __builtin_expect(!!(expr), 0)
-#  define likely(expr) __builtin_expect(!!(expr), 1)
+#  define unlikely(expr) (__builtin_expect((expr) ? 1 : 0, 0) != 0)
+#  define likely(expr) (__builtin_expect((expr) ? 1 : 0, 1) != 0)
 #else
 #  define unlikely(expr) expr
 #  define likely(expr) expr
@@ -193,39 +202,24 @@
 /* Provide macros for error handling. */
 #ifdef DISABLE_ASSERTS
 #  define i_assert(expr)
-#elif defined (__GNUC__) && !defined (__STRICT_ANSI__)
+#else
 
 #define i_assert(expr)			STMT_START{			\
      if (unlikely(!(expr)))						\
        i_panic("file %s: line %d (%s): assertion failed: (%s)",		\
 		__FILE__,						\
 		__LINE__,						\
-		__FUNCTION__,					\
+		__func__,					\
 		#expr);			}STMT_END
 
-#else /* !__GNUC__ */
-
-#define i_assert(expr)			STMT_START{		\
-     if (unlikely(!(expr)))					\
-       i_panic("file %s: line %d: assertion failed: (%s)",	\
-	      __FILE__,						\
-	      __LINE__,						\
-	      #expr);			}STMT_END
-
 #endif
 
-/* Close the fd and set it to -1. This assert-crashes if fd == 0. Normally
-   this would happen only if an uninitialized fd is attempted to be closed,
-   which is a bug. */
-#define i_close_fd(fd) STMT_START {  \
-	i_assert(*fd > 0); \
-	if (unlikely(close_keep_errno(fd) < 0)) \
-		i_error("close(%d[%s:%d]) failed: %m", \
-			*(fd), __FILE__, __LINE__); \
-	} STMT_END
-
-#define i_unreached() \
+#ifndef STATIC_CHECKER
+#  define i_unreached() \
 	i_panic("file %s: line %d: unreached", __FILE__, __LINE__)
+#else
+#  define i_unreached() __builtin_unreachable()
+#endif
 
 /* Convenience macros to test the versions of dovecot. */
 #if defined DOVECOT_VERSION_MAJOR && defined DOVECOT_VERSION_MINOR
@@ -241,7 +235,15 @@
 #endif
 
 /* Convenience wrappers for initializing a struct */
-#define i_zero(p) memset(p, 0, sizeof(*(p)))
-#define i_zero_safe(p) safe_memset(p, 0, sizeof(*(p)))
+#define i_zero(p) \
+	memset(p, 0 +  + COMPILE_ERROR_IF_TRUE(sizeof(p) > sizeof(void *)), sizeof(*(p)))
+#define i_zero_safe(p) \
+	safe_memset(p, 0 +  + COMPILE_ERROR_IF_TRUE(sizeof(p) > sizeof(void *)), sizeof(*(p)))
+
+#define ST_CHANGED(st_a, st_b) \
+	((st_a).st_mtime != (st_b).st_mtime || \
+	 ST_MTIME_NSEC(st_a) != ST_MTIME_NSEC(st_b) || \
+	 (st_a).st_size != (st_b).st_size || \
+	 (st_a).st_ino != (st_b).st_ino)
 
 #endif
