@@ -85,6 +85,8 @@ struct master_service {
 	bool ssl_ctx_initialized:1;
 	bool config_path_from_master:1;
 	bool log_initialized:1;
+	bool ssl_module_loaded:1;
+	bool init_finished:1;
 };
 
 void master_service_io_listeners_add(struct master_service *service);
